import type { Handler } from "aws-lambda";
import { createRestAPIClient } from "masto";
import "dotenv/config";

import ScraperArticles from "../../utils/scraper-articles";
import RedisClient from "../../libs/redis-client";

import config from "../../config";
import LogLevels from "../../enums/log-levels";
import Emojis from "../../enums/emojis";

export const handler: Handler = async (event, context) => {
  const name = "EMOL - El Mercurio Online";
  try {
    const redisClient = new RedisClient();
    const mastodon = createRestAPIClient({
      url: config.MASTODON_URL,
      accessToken: config.MASTODON_ACCESS_TOKEN
    });
    const scraperArticles = new ScraperArticles(name, {
      url: config.EMOL,
      articlesSelector: "div.cont_378_e_2015 div.col_center_noticias_item div.col_center_noticia4dest-360px",
      titleSelector: "h3 a",
      contentSelector: "h3 a",
      linkSelector: "h3 a",
      imageSelector: "div.contImgListadoNoticia a img",
      authorSelector: "",
      dateSelector: ""
    });

    const articles = await scraperArticles.getArticles();
    if (config.LOG_LEVEL === LogLevels.DEBUG) {
      console.log("Articles", articles);
    }

    let totalPublished = 0;
    const length = articles.length;

    // Order has to be reversed to appear in the correct order when posting
    for (let i = length - 1; i >= 0; i--) {
      const article = articles[i];
      const exists = await redisClient.retrieve(article.link);
      if (exists !== null) {
        continue;
      }

      const date = new Date(Date.now()).toLocaleDateString();
      const message = `${Emojis.NEWS} ${article.title}\nhttps:${article.link}`;
      const mediaIds = [""];

      if (message.trim().length === 0) {
        continue;
      }

      if (article.image !== null) {
        const media = await mastodon.v2.media.create({ file: article.image, description: article.title });
        mediaIds.push(media.id);
      }

      console.log("Sending", message);

      await mastodon.v1.statuses.create({ status: message, mediaIds });
      await redisClient.store(article.link, date, { EX: 60 * 60 * 24 }); // EX: 24 hrs expiration
      totalPublished++
    }
    console.log(`Published ${totalPublished} new articles`);
  } catch (err: any) {
    console.log('An error has occurred\n')
    console.error(err.message);
    if (config.LOG_LEVEL === LogLevels.DEBUG) {
      console.debug("\nEvent\n");
      console.debug(event);
      console.debug("\nContext\n");
      console.debug(context);
    }
  }

  return "The End.";
};
